@@ -8,37 +8,40 @@ over the internet, all details are available on the official website:
 # Copyright (c) 2011 Olivier Grisel <olivier.grisel@ensta.org>
 # License: BSD 3 clause
 
-from os import listdir, makedirs, remove
-from os.path import dirname, join, exists, isdir
-
 import logging
+from numbers import Integral, Real
+from os import listdir, makedirs, remove
+from os.path import exists, isdir, join
 
 import numpy as np
-import joblib
 from joblib import Memory
 
-from ._base import get_data_home, _fetch_remote, RemoteFileMetadata
 from ..utils import Bunch
-from ..utils.validation import _deprecate_positional_args
-from ..utils.fixes import parse_version
+from ..utils._param_validation import Hidden, Interval, StrOptions, validate_params
+from ._base import (
+    RemoteFileMetadata,
+    _fetch_remote,
+    get_data_home,
+    load_descr,
+)
 
 logger = logging.getLogger(__name__)
 
 # The original data can be found in:
 # http://vis-www.cs.umass.edu/lfw/lfw.tgz
 ARCHIVE = RemoteFileMetadata(
-    filename='lfw.tgz',
-    url='https://ndownloader.figshare.com/files/5976018',
-    checksum=('055f7d9c632d7370e6fb4afc7468d40f'
-              '970c34a80d4c6f50ffec63f5a8d536c0'))
+    filename="lfw.tgz",
+    url="https://ndownloader.figshare.com/files/5976018",
+    checksum="055f7d9c632d7370e6fb4afc7468d40f970c34a80d4c6f50ffec63f5a8d536c0",
+)
 
 # The original funneled data can be found in:
 # http://vis-www.cs.umass.edu/lfw/lfw-funneled.tgz
 FUNNELED_ARCHIVE = RemoteFileMetadata(
-    filename='lfw-funneled.tgz',
-    url='https://ndownloader.figshare.com/files/5976015',
-    checksum=('b47c8422c8cded889dc5a13418c4bc2a'
-              'bbda121092b3533a83306f90d900100a'))
+    filename="lfw-funneled.tgz",
+    url="https://ndownloader.figshare.com/files/5976015",
+    checksum="b47c8422c8cded889dc5a13418c4bc2abbda121092b3533a83306f90d900100a",
+)
 
 # The original target data can be found in:
 # http://vis-www.cs.umass.edu/lfw/pairsDevTrain.txt',
@@ -46,22 +49,20 @@ FUNNELED_ARCHIVE = RemoteFileMetadata(
 # http://vis-www.cs.umass.edu/lfw/pairs.txt',
 TARGETS = (
     RemoteFileMetadata(
-        filename='pairsDevTrain.txt',
-        url='https://ndownloader.figshare.com/files/5976012',
-        checksum=('1d454dada7dfeca0e7eab6f65dc4e97a'
-                  '6312d44cf142207be28d688be92aabfa')),
-
+        filename="pairsDevTrain.txt",
+        url="https://ndownloader.figshare.com/files/5976012",
+        checksum="1d454dada7dfeca0e7eab6f65dc4e97a6312d44cf142207be28d688be92aabfa",
+    ),
     RemoteFileMetadata(
-        filename='pairsDevTest.txt',
-        url='https://ndownloader.figshare.com/files/5976009',
-        checksum=('7cb06600ea8b2814ac26e946201cdb30'
-                  '4296262aad67d046a16a7ec85d0ff87c')),
-
+        filename="pairsDevTest.txt",
+        url="https://ndownloader.figshare.com/files/5976009",
+        checksum="7cb06600ea8b2814ac26e946201cdb304296262aad67d046a16a7ec85d0ff87c",
+    ),
     RemoteFileMetadata(
-        filename='pairs.txt',
-        url='https://ndownloader.figshare.com/files/5976006',
-        checksum=('ea42330c62c92989f9d7c03237ed5d59'
-                  '1365e89b3e649747777b70e692dc1592')),
+        filename="pairs.txt",
+        url="https://ndownloader.figshare.com/files/5976006",
+        checksum="ea42330c62c92989f9d7c03237ed5d591365e89b3e649747777b70e692dc1592",
+    ),
 )
 
 
@@ -87,7 +88,7 @@ def _check_fetch_lfw(data_home=None, funneled=True, download_if_missing=True):
                 logger.info("Downloading LFW metadata: %s", target.url)
                 _fetch_remote(target, dirname=lfw_home)
             else:
-                raise IOError("%s is missing" % target_filepath)
+                raise OSError("%s is missing" % target_filepath)
 
     if funneled:
         data_folder_path = join(lfw_home, "lfw_funneled")
@@ -100,13 +101,13 @@ def _check_fetch_lfw(data_home=None, funneled=True, download_if_missing=True):
         archive_path = join(lfw_home, archive.filename)
         if not exists(archive_path):
             if download_if_missing:
-                logger.info("Downloading LFW data (~200MB): %s",
-                            archive.url)
+                logger.info("Downloading LFW data (~200MB): %s", archive.url)
                 _fetch_remote(archive, dirname=lfw_home)
             else:
-                raise IOError("%s is missing" % archive_path)
+                raise OSError("%s is missing" % archive_path)
 
         import tarfile
+
         logger.debug("Decompressing the data archive to %s", data_folder_path)
         tarfile.open(archive_path, "r:gz").extractall(path=lfw_home)
         remove(archive_path)
@@ -116,8 +117,15 @@ def _check_fetch_lfw(data_home=None, funneled=True, download_if_missing=True):
 
 def _load_imgs(file_paths, slice_, color, resize):
     """Internally used to load images"""
-    # import PIL only when needed
-    from ..externals._pilutil import imread, imresize
+    try:
+        from PIL import Image
+    except ImportError:
+        raise ImportError(
+            "The Python Imaging Library (PIL) is required to load data "
+            "from jpeg files. Please refer to "
+            "https://pillow.readthedocs.io/en/stable/installation.html "
+            "for installing PIL."
+        )
 
     # compute the portion of the images to load to respect the slice_ parameter
     # given by the caller
@@ -151,16 +159,21 @@ def _load_imgs(file_paths, slice_, color, resize):
 
         # Checks if jpeg reading worked. Refer to issue #3594 for more
         # details.
-        img = imread(file_path)
-        if img.ndim == 0:
-            raise RuntimeError("Failed to read the image file %s, "
-                               "Please make sure that libjpeg is installed"
-                               % file_path)
-
-        face = np.asarray(img[slice_], dtype=np.float32)
-        face /= 255.0  # scale uint8 coded colors to the [0.0, 1.0] floats
+        pil_img = Image.open(file_path)
+        pil_img = pil_img.crop(
+            (w_slice.start, h_slice.start, w_slice.stop, h_slice.stop)
+        )
         if resize is not None:
-            face = imresize(face, resize)
+            pil_img = pil_img.resize((w, h))
+        face = np.asarray(pil_img, dtype=np.float32)
+
+        if face.ndim == 0:
+            raise RuntimeError(
+                "Failed to read the image file %s, "
+                "Please make sure that libjpeg is installed" % file_path
+            )
+
+        face /= 255.0  # scale uint8 coded colors to the [0.0, 1.0] floats
         if not color:
             # average the color channels to compute a gray levels
             # representation
@@ -175,8 +188,10 @@ def _load_imgs(file_paths, slice_, color, resize):
 # Task #1:  Face Identification on picture with names
 #
 
-def _fetch_lfw_people(data_folder_path, slice_=None, color=False, resize=None,
-                      min_faces_per_person=0):
+
+def _fetch_lfw_people(
+    data_folder_path, slice_=None, color=False, resize=None, min_faces_per_person=0
+):
     """Perform the actual data loading for the lfw people dataset
 
     This operation is meant to be cached by a joblib wrapper.
@@ -191,14 +206,15 @@ def _fetch_lfw_people(data_folder_path, slice_=None, color=False, resize=None,
         paths = [join(folder_path, f) for f in sorted(listdir(folder_path))]
         n_pictures = len(paths)
         if n_pictures >= min_faces_per_person:
-            person_name = person_name.replace('_', ' ')
+            person_name = person_name.replace("_", " ")
             person_names.extend([person_name] * n_pictures)
             file_paths.extend(paths)
 
     n_faces = len(file_paths)
     if n_faces == 0:
-        raise ValueError("min_faces_per_person=%d is too restrictive" %
-                         min_faces_per_person)
+        raise ValueError(
+            "min_faces_per_person=%d is too restrictive" % min_faces_per_person
+        )
 
     target_names = np.unique(person_names)
     target = np.searchsorted(target_names, person_names)
@@ -216,11 +232,30 @@ def _fetch_lfw_people(data_folder_path, slice_=None, color=False, resize=None,
     return faces, target, target_names
 
 
-@_deprecate_positional_args
-def fetch_lfw_people(*, data_home=None, funneled=True, resize=0.5,
-                     min_faces_per_person=0, color=False,
-                     slice_=(slice(70, 195), slice(78, 172)),
-                     download_if_missing=True, return_X_y=False):
+@validate_params(
+    {
+        "data_home": [str, None],
+        "funneled": ["boolean"],
+        "resize": [Interval(Real, 0, None, closed="neither"), None],
+        "min_faces_per_person": [Interval(Integral, 0, None, closed="left"), None],
+        "color": ["boolean"],
+        "slice_": [tuple, Hidden(None)],
+        "download_if_missing": ["boolean"],
+        "return_X_y": ["boolean"],
+    },
+    prefer_skip_nested_validation=True,
+)
+def fetch_lfw_people(
+    *,
+    data_home=None,
+    funneled=True,
+    resize=0.5,
+    min_faces_per_person=0,
+    color=False,
+    slice_=(slice(70, 195), slice(78, 172)),
+    download_if_missing=True,
+    return_X_y=False,
+):
     """Load the Labeled Faces in the Wild (LFW) people dataset \
 (classification).
 
@@ -244,8 +279,9 @@ def fetch_lfw_people(*, data_home=None, funneled=True, resize=0.5,
     funneled : bool, default=True
         Download and use the funneled variant of the dataset.
 
-    resize : float, default=0.5
-        Ratio used to resize the each face picture.
+    resize : float or None, default=0.5
+        Ratio used to resize the each face picture. If `None`, no resizing is
+        performed.
 
     min_faces_per_person : int, default=None
         The extracted dataset will only retain pictures of people that have at
@@ -259,10 +295,10 @@ def fetch_lfw_people(*, data_home=None, funneled=True, resize=0.5,
     slice_ : tuple of slice, default=(slice(70, 195), slice(78, 172))
         Provide a custom 2D slice (height, width) to extract the
         'interesting' part of the jpeg files and avoid use statistical
-        correlation from the background
+        correlation from the background.
 
     download_if_missing : bool, default=True
-        If False, raise a IOError if the data is not locally available
+        If False, raise an OSError if the data is not locally available
         instead of trying to download the data from the source site.
 
     return_X_y : bool, default=False
@@ -289,46 +325,50 @@ def fetch_lfw_people(*, data_home=None, funneled=True, resize=0.5,
         target : numpy array of shape (13233,)
             Labels associated to each face image.
             Those labels range from 0-5748 and correspond to the person IDs.
-        DESCR : string
+        target_names : numpy array of shape (5749,)
+            Names of all persons in the dataset.
+            Position in array corresponds to the person ID in the target array.
+        DESCR : str
             Description of the Labeled Faces in the Wild (LFW) dataset.
 
     (data, target) : tuple if ``return_X_y`` is True
+        A tuple of two ndarray. The first containing a 2D array of
+        shape (n_samples, n_features) with each row representing one
+        sample and each column representing the features. The second
+        ndarray of shape (n_samples,) containing the target samples.
 
         .. versionadded:: 0.20
-
     """
     lfw_home, data_folder_path = _check_fetch_lfw(
-        data_home=data_home, funneled=funneled,
-        download_if_missing=download_if_missing)
-    logger.debug('Loading LFW people faces from %s', lfw_home)
+        data_home=data_home, funneled=funneled, download_if_missing=download_if_missing
+    )
+    logger.debug("Loading LFW people faces from %s", lfw_home)
 
     # wrap the loader in a memoizing function that will return memmaped data
     # arrays for optimal memory usage
-    if parse_version(joblib.__version__) < parse_version('0.12'):
-        # Deal with change of API in joblib
-        m = Memory(cachedir=lfw_home, compress=6, verbose=0)
-    else:
-        m = Memory(location=lfw_home, compress=6, verbose=0)
+    m = Memory(location=lfw_home, compress=6, verbose=0)
     load_func = m.cache(_fetch_lfw_people)
 
     # load and memoize the pairs as np arrays
     faces, target, target_names = load_func(
-        data_folder_path, resize=resize,
-        min_faces_per_person=min_faces_per_person, color=color, slice_=slice_)
+        data_folder_path,
+        resize=resize,
+        min_faces_per_person=min_faces_per_person,
+        color=color,
+        slice_=slice_,
+    )
 
     X = faces.reshape(len(faces), -1)
 
-    module_path = dirname(__file__)
-    with open(join(module_path, 'descr', 'lfw.rst')) as rst_file:
-        fdescr = rst_file.read()
+    fdescr = load_descr("lfw.rst")
 
     if return_X_y:
         return X, target
 
     # pack the results as a Bunch instance
-    return Bunch(data=X, images=faces,
-                 target=target, target_names=target_names,
-                 DESCR=fdescr)
+    return Bunch(
+        data=X, images=faces, target=target, target_names=target_names, DESCR=fdescr
+    )
 
 
 #
@@ -336,16 +376,17 @@ def fetch_lfw_people(*, data_home=None, funneled=True, resize=0.5,
 #
 
 
-def _fetch_lfw_pairs(index_file_path, data_folder_path, slice_=None,
-                     color=False, resize=None):
+def _fetch_lfw_pairs(
+    index_file_path, data_folder_path, slice_=None, color=False, resize=None
+):
     """Perform the actual data loading for the LFW pairs dataset
 
     This operation is meant to be cached by a joblib wrapper.
     """
     # parse the index file to find the number of pairs to be able to allocate
     # the right amount of memory before starting to decode the jpeg files
-    with open(index_file_path, 'rb') as index_file:
-        split_lines = [ln.decode().strip().split('\t') for ln in index_file]
+    with open(index_file_path, "rb") as index_file:
+        split_lines = [ln.decode().strip().split("\t") for ln in index_file]
     pair_specs = [sl for sl in split_lines if len(sl) > 2]
     n_pairs = len(pair_specs)
 
@@ -372,7 +413,7 @@ def _fetch_lfw_pairs(index_file_path, data_folder_path, slice_=None,
             try:
                 person_folder = join(data_folder_path, name)
             except TypeError:
-                person_folder = join(data_folder_path, str(name, 'UTF-8'))
+                person_folder = join(data_folder_path, str(name, "UTF-8"))
             filenames = list(sorted(listdir(person_folder)))
             file_path = join(person_folder, filenames[idx])
             file_paths.append(file_path)
@@ -384,14 +425,31 @@ def _fetch_lfw_pairs(index_file_path, data_folder_path, slice_=None,
     shape.insert(0, n_faces // 2)
     pairs.shape = shape
 
-    return pairs, target, np.array(['Different persons', 'Same person'])
+    return pairs, target, np.array(["Different persons", "Same person"])
 
 
-@_deprecate_positional_args
-def fetch_lfw_pairs(*, subset='train', data_home=None, funneled=True,
-                    resize=0.5,
-                    color=False, slice_=(slice(70, 195), slice(78, 172)),
-                    download_if_missing=True):
+@validate_params(
+    {
+        "subset": [StrOptions({"train", "test", "10_folds"})],
+        "data_home": [str, None],
+        "funneled": ["boolean"],
+        "resize": [Interval(Real, 0, None, closed="neither"), None],
+        "color": ["boolean"],
+        "slice_": [tuple, Hidden(None)],
+        "download_if_missing": ["boolean"],
+    },
+    prefer_skip_nested_validation=True,
+)
+def fetch_lfw_pairs(
+    *,
+    subset="train",
+    data_home=None,
+    funneled=True,
+    resize=0.5,
+    color=False,
+    slice_=(slice(70, 195), slice(78, 172)),
+    download_if_missing=True,
+):
     """Load the Labeled Faces in the Wild (LFW) pairs dataset (classification).
 
     Download it if necessary.
@@ -441,10 +499,10 @@ def fetch_lfw_pairs(*, subset='train', data_home=None, funneled=True,
     slice_ : tuple of slice, default=(slice(70, 195), slice(78, 172))
         Provide a custom 2D slice (height, width) to extract the
         'interesting' part of the jpeg files and avoid use statistical
-        correlation from the background
+        correlation from the background.
 
     download_if_missing : bool, default=True
-        If False, raise a IOError if the data is not locally available
+        If False, raise an OSError if the data is not locally available
         instead of trying to download the data from the source site.
 
     Returns
@@ -466,45 +524,47 @@ def fetch_lfw_pairs(*, subset='train', data_home=None, funneled=True,
         target : numpy array of shape (2200,). Shape depends on ``subset``.
             Labels associated to each pair of images.
             The two label values being different persons or the same person.
-        DESCR : string
+        target_names : numpy array of shape (2,)
+            Explains the target values of the target array.
+            0 corresponds to "Different person", 1 corresponds to "same person".
+        DESCR : str
             Description of the Labeled Faces in the Wild (LFW) dataset.
-
     """
     lfw_home, data_folder_path = _check_fetch_lfw(
-        data_home=data_home, funneled=funneled,
-        download_if_missing=download_if_missing)
-    logger.debug('Loading %s LFW pairs from %s', subset, lfw_home)
+        data_home=data_home, funneled=funneled, download_if_missing=download_if_missing
+    )
+    logger.debug("Loading %s LFW pairs from %s", subset, lfw_home)
 
     # wrap the loader in a memoizing function that will return memmaped data
     # arrays for optimal memory usage
-    if parse_version(joblib.__version__) < parse_version('0.12'):
-        # Deal with change of API in joblib
-        m = Memory(cachedir=lfw_home, compress=6, verbose=0)
-    else:
-        m = Memory(location=lfw_home, compress=6, verbose=0)
+    m = Memory(location=lfw_home, compress=6, verbose=0)
     load_func = m.cache(_fetch_lfw_pairs)
 
     # select the right metadata file according to the requested subset
     label_filenames = {
-        'train': 'pairsDevTrain.txt',
-        'test': 'pairsDevTest.txt',
-        '10_folds': 'pairs.txt',
+        "train": "pairsDevTrain.txt",
+        "test": "pairsDevTest.txt",
+        "10_folds": "pairs.txt",
     }
     if subset not in label_filenames:
-        raise ValueError("subset='%s' is invalid: should be one of %r" % (
-            subset, list(sorted(label_filenames.keys()))))
+        raise ValueError(
+            "subset='%s' is invalid: should be one of %r"
+            % (subset, list(sorted(label_filenames.keys())))
+        )
     index_file_path = join(lfw_home, label_filenames[subset])
 
     # load and memoize the pairs as np arrays
     pairs, target, target_names = load_func(
-        index_file_path, data_folder_path, resize=resize, color=color,
-        slice_=slice_)
+        index_file_path, data_folder_path, resize=resize, color=color, slice_=slice_
+    )
 
-    module_path = dirname(__file__)
-    with open(join(module_path, 'descr', 'lfw.rst')) as rst_file:
-        fdescr = rst_file.read()
+    fdescr = load_descr("lfw.rst")
 
     # pack the results as a Bunch instance
-    return Bunch(data=pairs.reshape(len(pairs), -1), pairs=pairs,
-                 target=target, target_names=target_names,
-                 DESCR=fdescr)
+    return Bunch(
+        data=pairs.reshape(len(pairs), -1),
+        pairs=pairs,
+        target=target,
+        target_names=target_names,
+        DESCR=fdescr,
+    )
