@@ -2,16 +2,14 @@
 decision trees.
 """
 
-from abc import ABCMeta
-from abc import abstractmethod
+from abc import ABCMeta, abstractmethod
 
 import numpy as np
 from scipy.special import expit, logsumexp
 
+from ..dummy import DummyClassifier, DummyRegressor
 from ..tree._tree import TREE_LEAF
 from ..utils.stats import _weighted_percentile
-from ..dummy import DummyClassifier
-from ..dummy import DummyRegressor
 
 
 class LossFunction(metaclass=ABCMeta):
@@ -35,9 +33,9 @@ class LossFunction(metaclass=ABCMeta):
     def __init__(self, n_classes):
         self.K = n_classes
 
+    @abstractmethod
     def init_estimator(self):
         """Default ``init`` estimator for loss function."""
-        raise NotImplementedError()
 
     @abstractmethod
     def __call__(self, y, raw_predictions, sample_weight=None):
@@ -584,6 +582,7 @@ class QuantileLossFunction(RegressionLossFunction):
 class ClassificationLossFunction(LossFunction, metaclass=ABCMeta):
     """Base class for classification loss functions."""
 
+    @abstractmethod
     def _raw_prediction_to_proba(self, raw_predictions):
         """Template method to convert raw predictions into probabilities.
 
@@ -986,15 +985,11 @@ class ExponentialLoss(ClassificationLossFunction):
         return raw_predictions.reshape(-1, 1).astype(np.float64)
 
 
-# TODO: Remove entry 'ls' and 'lad' in version 1.2.
 LOSS_FUNCTIONS = {
     "squared_error": LeastSquaresError,
-    "ls": LeastSquaresError,
     "absolute_error": LeastAbsoluteError,
-    "lad": LeastAbsoluteError,
     "huber": HuberLossFunction,
     "quantile": QuantileLossFunction,
-    "deviance": None,  # for both, multinomial and binomial
     "log_loss": None,  # for both, multinomial and binomial
     "exponential": ExponentialLoss,
 }
