@@ -2,21 +2,25 @@
 Testing for the gradient boosting loss functions and initial estimators.
 """
 from itertools import product
+
 import numpy as np
-from numpy.testing import assert_allclose
 import pytest
+from numpy.testing import assert_allclose
 from pytest import approx
 
+from sklearn.ensemble._gb_losses import (
+    LOSS_FUNCTIONS,
+    BinomialDeviance,
+    ExponentialLoss,
+    HuberLossFunction,
+    LeastAbsoluteError,
+    LeastSquaresError,
+    MultinomialDeviance,
+    QuantileLossFunction,
+    RegressionLossFunction,
+)
+from sklearn.metrics import mean_pinball_loss
 from sklearn.utils import check_random_state
-from sklearn.ensemble._gb_losses import RegressionLossFunction
-from sklearn.ensemble._gb_losses import LeastSquaresError
-from sklearn.ensemble._gb_losses import LeastAbsoluteError
-from sklearn.ensemble._gb_losses import HuberLossFunction
-from sklearn.ensemble._gb_losses import QuantileLossFunction
-from sklearn.ensemble._gb_losses import BinomialDeviance
-from sklearn.ensemble._gb_losses import MultinomialDeviance
-from sklearn.ensemble._gb_losses import ExponentialLoss
-from sklearn.ensemble._gb_losses import LOSS_FUNCTIONS
 
 
 def test_binomial_deviance():
@@ -25,11 +29,10 @@ def test_binomial_deviance():
     bd = BinomialDeviance(2)
 
     # pred has the same BD for y in {0, 1}
-    assert (bd(np.array([0.]), np.array([0.])) ==
-            bd(np.array([1.]), np.array([0.])))
+    assert bd(np.array([0.0]), np.array([0.0])) == bd(np.array([1.0]), np.array([0.0]))
 
-    assert bd(np.array([1., 1, 1]), np.array([100., 100, 100])) == approx(0)
-    assert bd(np.array([1., 0, 0]), np.array([100., -100, -100])) == approx(0)
+    assert bd(np.array([1.0, 1, 1]), np.array([100.0, 100, 100])) == approx(0)
+    assert bd(np.array([1.0, 0, 0]), np.array([100.0, -100, -100])) == approx(0)
 
     # check if same results as alternative definition of deviance, from ESLII
     # Eq. (10.18): -loglike = log(1 + exp(-2*z*f))
@@ -42,13 +45,14 @@ def test_binomial_deviance():
         return 2 * np.mean(np.log(1 + np.exp(-z * raw_pred)))
 
     test_data = product(
-        (np.array([0., 0, 0]), np.array([1., 1, 1])),
-        (np.array([-5., -5, -5]), np.array([3., 3, 3])))
+        (np.array([0.0, 0, 0]), np.array([1.0, 1, 1])),
+        (np.array([-5.0, -5, -5]), np.array([3.0, 3, 3])),
+    )
 
     for datum in test_data:
         assert bd(*datum) == approx(alt_dev(*datum))
 
-    # check the negative gradient against altenative formula from ESLII
+    # check the negative gradient against alternative formula from ESLII
     # Note: negative_gradient is half the negative gradient.
     def alt_ng(y, raw_pred):
         z = 2 * y - 1
@@ -115,6 +119,8 @@ def test_quantile_loss_function():
     y_found = QuantileLossFunction(0.9)(x, np.zeros_like(x))
     y_expected = np.asarray([0.1, 0.0, 0.9]).mean()
     np.testing.assert_allclose(y_found, y_expected)
+    y_found_p = mean_pinball_loss(x, np.zeros_like(x), alpha=0.9)
+    np.testing.assert_allclose(y_found, y_found_p)
 
 
 def test_sample_weight_deviance():
@@ -147,15 +153,13 @@ def test_sample_weight_deviance():
 
         deviance_w_w = loss(y, p, sample_weight)
         deviance_wo_w = loss(y, p)
-        assert deviance_wo_w == deviance_w_w
+        assert_allclose(deviance_wo_w, deviance_w_w)
 
 
-@pytest.mark.parametrize(
-    'n_classes, n_samples', [(3, 100), (5, 57), (7, 13)]
-)
-def test_multinomial_deviance(n_classes, n_samples):
+@pytest.mark.parametrize("n_classes, n_samples", [(3, 100), (5, 57), (7, 13)])
+def test_multinomial_deviance(n_classes, n_samples, global_random_seed):
     # Check multinomial deviance with and without sample weights.
-    rng = np.random.RandomState(13)
+    rng = np.random.RandomState(global_random_seed)
     sample_weight = np.ones(n_samples)
     y_true = rng.randint(0, n_classes, size=n_samples)
     y_pred = np.zeros((n_samples, n_classes), dtype=np.float64)
@@ -176,7 +180,7 @@ def test_multinomial_deviance(n_classes, n_samples):
 
 
 def test_mdl_computation_weighted():
-    raw_predictions = np.array([[1., -1., -.1], [-2., 1., 2.]])
+    raw_predictions = np.array([[1.0, -1.0, -0.1], [-2.0, 1.0, 2.0]])
     y_true = np.array([0, 1])
     weights = np.array([1, 3])
     expected_loss = 1.0909323
@@ -185,10 +189,10 @@ def test_mdl_computation_weighted():
     assert loss(y_true, raw_predictions, weights) == approx(expected_loss)
 
 
-@pytest.mark.parametrize('n', [0, 1, 2])
+@pytest.mark.parametrize("n", [0, 1, 2])
 def test_mdl_exception(n):
     # Check that MultinomialDeviance throws an exception when n_classes <= 2
-    err_msg = 'MultinomialDeviance requires more than 2 classes.'
+    err_msg = "MultinomialDeviance requires more than 2 classes."
     with pytest.raises(ValueError, match=err_msg):
         MultinomialDeviance(n)
 
@@ -202,18 +206,19 @@ def test_init_raw_predictions_shapes():
     n_samples = 100
     X = rng.normal(size=(n_samples, 5))
     y = rng.normal(size=n_samples)
-    for loss in (LeastSquaresError(),
-                 LeastAbsoluteError(),
-                 QuantileLossFunction(),
-                 HuberLossFunction()):
+    for loss in (
+        LeastSquaresError(),
+        LeastAbsoluteError(),
+        QuantileLossFunction(),
+        HuberLossFunction(),
+    ):
         init_estimator = loss.init_estimator().fit(X, y)
         raw_predictions = loss.get_init_raw_predictions(y, init_estimator)
         assert raw_predictions.shape == (n_samples, 1)
         assert raw_predictions.dtype == np.float64
 
     y = rng.randint(0, 2, size=n_samples)
-    for loss in (BinomialDeviance(n_classes=2),
-                 ExponentialLoss(n_classes=2)):
+    for loss in (BinomialDeviance(n_classes=2), ExponentialLoss(n_classes=2)):
         init_estimator = loss.init_estimator().fit(X, y)
         raw_predictions = loss.get_init_raw_predictions(y, init_estimator)
         assert raw_predictions.shape == (n_samples, 1)
@@ -228,10 +233,10 @@ def test_init_raw_predictions_shapes():
         assert raw_predictions.dtype == np.float64
 
 
-def test_init_raw_predictions_values():
+def test_init_raw_predictions_values(global_random_seed):
     # Make sure the get_init_raw_predictions() returns the expected values for
     # each loss.
-    rng = np.random.RandomState(0)
+    rng = np.random.RandomState(global_random_seed)
 
     n_samples = 100
     X = rng.normal(size=(n_samples, 5))
@@ -253,7 +258,7 @@ def test_init_raw_predictions_values():
         assert_allclose(raw_predictions, np.median(y))
 
     # Quantile loss
-    for alpha in (.1, .5, .9):
+    for alpha in (0.1, 0.5, 0.9):
         loss = QuantileLossFunction(alpha=alpha)
         init_estimator = loss.init_estimator().fit(X, y)
         raw_predictions = loss.get_init_raw_predictions(y, init_estimator)
@@ -279,7 +284,7 @@ def test_init_raw_predictions_values():
     init_estimator = loss.init_estimator().fit(X, y)
     raw_predictions = loss.get_init_raw_predictions(y, init_estimator)
     p = y.mean()
-    assert_allclose(raw_predictions, .5 * np.log(p / (1 - p)))
+    assert_allclose(raw_predictions, 0.5 * np.log(p / (1 - p)))
 
     # Multinomial deviance loss
     for n_classes in range(3, 5):
@@ -292,22 +297,43 @@ def test_init_raw_predictions_values():
             assert_allclose(raw_predictions[:, k], np.log(p))
 
 
-@pytest.mark.parametrize('seed', range(5))
-def test_lad_equals_quantile_50(seed):
+@pytest.mark.parametrize("alpha", [0.4, 0.5, 0.6])
+def test_lad_equals_quantiles(global_random_seed, alpha):
     # Make sure quantile loss with alpha = .5 is equivalent to LAD
     lad = LeastAbsoluteError()
-    ql = QuantileLossFunction(alpha=0.5)
+    ql = QuantileLossFunction(alpha=alpha)
 
     n_samples = 50
-    rng = np.random.RandomState(seed)
+    rng = np.random.RandomState(global_random_seed)
     raw_predictions = rng.normal(size=(n_samples))
     y_true = rng.normal(size=(n_samples))
 
     lad_loss = lad(y_true, raw_predictions)
     ql_loss = ql(y_true, raw_predictions)
-    assert lad_loss == approx(2 * ql_loss)
+    if alpha == 0.5:
+        assert lad_loss == approx(2 * ql_loss)
 
     weights = np.linspace(0, 1, n_samples) ** 2
     lad_weighted_loss = lad(y_true, raw_predictions, sample_weight=weights)
     ql_weighted_loss = ql(y_true, raw_predictions, sample_weight=weights)
-    assert lad_weighted_loss == approx(2 * ql_weighted_loss)
+    if alpha == 0.5:
+        assert lad_weighted_loss == approx(2 * ql_weighted_loss)
+    pbl_weighted_loss = mean_pinball_loss(
+        y_true, raw_predictions, sample_weight=weights, alpha=alpha
+    )
+    assert pbl_weighted_loss == approx(ql_weighted_loss)
+
+
+def test_exponential_loss():
+    """Check that we compute the negative gradient of the exponential loss.
+
+    Non-regression test for:
+    https://github.com/scikit-learn/scikit-learn/issues/9666
+    """
+    loss = ExponentialLoss(n_classes=2)
+    y_true = np.array([0])
+    y_pred = np.array([0])
+    # we expect to have loss = exp(0) = 1
+    assert loss(y_true, y_pred) == pytest.approx(1)
+    # we expect to have negative gradient = -1 * (1 * exp(0)) = -1
+    assert_allclose(loss.negative_gradient(y_true, y_pred), -1)
